package djmil.cordacheckers;

import org.springframework.security.core.userdetails.User;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;

import djmil.cordacheckers.cordaclient.HoldingIdentityResolver;
import djmil.cordacheckers.cordaclient.dao.HoldingIdentity;

@Service
public class ApiUserDetailsService implements UserDetailsService {

    private final PasswordEncoder encoder;
    private final HoldingIdentityResolver holdingIdentityResolver;

    public ApiUserDetailsService(
		    PasswordEncoder encoder,
		    HoldingIdentityResolver holdingIdentityNameResolver) {
        this.encoder = encoder;
        this.holdingIdentityResolver = holdingIdentityNameResolver;
    }

    @Override
    public ApiUserDetails loadUserByUsername(String username) throws UsernameNotFoundException {
        HoldingIdentity holdingIdentity = holdingIdentityResolver.getByCommonName(username);
        if (holdingIdentity == null) {
            throw new UsernameNotFoundException("Can't find HoldingIdentity for the user '"+username+ "'");
        }

        System.out.println("Load user "+username);
        
        User.UserBuilder userBuilder = User.builder();
        UserDetails user = userBuilder
            .username(username)
            .password(encoder.encode("qaz123"))
            .build();
        
        return new ApiUserDetails(user, holdingIdentity);
    }
}
