package djmil.cordacheckers.contracts;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import djmil.cordacheckers.states.GameProposalState;
import net.corda.v5.base.exceptions.CordaRuntimeException;
import net.corda.v5.ledger.utxo.Command;
import net.corda.v5.ledger.utxo.transaction.UtxoLedgerTransaction;

public class GameProposalContract implements net.corda.v5.ledger.utxo.Contract {

    private final static Logger log = LoggerFactory.getLogger(GameProposalContract.class);

    public static class Create implements Command { }
    public static class Accept implements Command { }
    public static class Reject implements Command { }
    public static class Cancle implements Command { }

    @Override
    public void verify(UtxoLedgerTransaction trx) {
        log.info("GameProposalContract.verify() called");

        requireThat(trx.getCommands().size() == 1, REQUIRE_SINGLE_COMMAND);
        Command command = trx.getCommands().get(0);

        if (command instanceof Create) {
            requireThat(trx.getInputContractStates().isEmpty(), CREATE_INPUT_STATE);
            requireThat(trx.getOutputContractStates().size() == 1, CREATE_OUTPUT_STATE);
            
            GameProposalState outputState = trx.getOutputStates(GameProposalState.class).get(0);
            requireThat(outputState != null, CREATE_OUTPUT_STATE);

            requireThat(outputState.getYouPlayAs() != null, CREATE_NOT_NULL_YOU_PLAY_AS);
        } else 
        if (command instanceof Accept) {
            // TODO outputState -> Game
            throw new CordaRuntimeException("Unimplemented!");
        } else 
        if (command instanceof Reject) {
            requireThat(trx.getInputContractStates().size() == 1, REJECT_INPUT_STATE);
            requireThat(trx.getOutputContractStates().isEmpty(), REJECT_OUTPUT_STATE);

            GameProposalState inputState = trx.getInputStates(GameProposalState.class).get(0);
            requireThat(inputState != null, REJECT_INPUT_STATE);
        } else 
        if (command instanceof Cancle) {
            // TODO cancle game state
            throw new CordaRuntimeException("Unimplemented!");
        } else {
            throw new CordaRuntimeException(UNKNOWN_COMMAND);
        }
    }
    
    private void requireThat(boolean asserted, String errorMessage) {
        if(!asserted) {
            throw new CordaRuntimeException("Failed requirement: " + errorMessage);
        }
    }

    static final String REQUIRE_SINGLE_COMMAND = "Require a single command";
    static final String UNKNOWN_COMMAND = "Unsupported command";

    static final String CREATE_INPUT_STATE  = "Create command should have no input states";
    static final String CREATE_OUTPUT_STATE = "Create command should output exactly one GameProposal state";
    static final String CREATE_NOT_NULL_YOU_PLAY_AS = "GameProposal.youPlayAs field can not be null";

    static final String REJECT_INPUT_STATE  = "Reject command should have exactly one GameProposal state";
    static final String REJECT_OUTPUT_STATE = "Reject command should have no output states";

}
