package djmil.cordacheckers.states;

import java.security.PublicKey;
import java.util.List;
import java.util.UUID;

import djmil.cordacheckers.contracts.GameProposalContract;
import net.corda.v5.base.annotations.ConstructorForDeserialization;
import net.corda.v5.base.annotations.CordaSerializable;
import net.corda.v5.base.types.MemberX500Name;
import net.corda.v5.ledger.utxo.BelongsToContract;
import net.corda.v5.ledger.utxo.ContractState;

@BelongsToContract(GameProposalContract.class)
public class GameProposalState implements ContractState {

    @CordaSerializable
    public enum Color {
        WHITE,
        BLACK,
    }
      
    public final MemberX500Name sender;
    public final MemberX500Name recipient;
    public final Color youPlayAs;
    public final String additionalMessage;
    public final UUID id;
    public final List<PublicKey> participants;

    // Allows serialisation and to use a specified UUID
    @ConstructorForDeserialization
    public GameProposalState(
                    MemberX500Name sender,
                    MemberX500Name recipient,
                    Color youPlayAs,
                    String additionalMessage,
                    UUID id,
                    List<PublicKey> participants) {
        this.sender = sender;
        this.recipient = recipient;
        this.youPlayAs = youPlayAs;
        this.additionalMessage = additionalMessage;
        this.id = id;
        this.participants = participants;
    }

    public MemberX500Name getSender() {
        return sender;
    }

    public MemberX500Name getRecipient() {
        return recipient;
    }

    public Color getYouPlayAs() {
        return youPlayAs;
    }

    public String getAdditionalMessage() {
        return additionalMessage;
    }

    public UUID getId() {
        return id;
    }

    public List<PublicKey> getParticipants() {
        return participants;
    }

    public String getRecipientCommonName() {
        return recipient == null ? "" : recipient.getCommonName();
    }

    public String getSenderCommonName() {
        return sender == null ? "" : sender.getCommonName();
    }
}
