package djmil.cordacheckers.cordaclient;

import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import javax.naming.InvalidNameException;

import org.springframework.stereotype.Service;

import djmil.cordacheckers.cordaclient.dao.HoldingIdentity;
import djmil.cordacheckers.cordaclient.dao.VirtualNode;

@Service
public class HoldingIdentityResolver {
    static final Locale locale = Locale.getDefault();

    /*
     * NOTE: Proof-of-Concept impementation 
     *  Ideally, we want to be able to update HoldingIdentity cache, had the 
     *  corda cluster configuration changed (aka in case of a cache miss)
     */ 
    final Map<String, HoldingIdentity> cache;

    HoldingIdentityResolver(CordaClient cordaClient) {
        this.cache = setCache(cordaClient);
    }

    private static Map<String, HoldingIdentity> setCache(CordaClient cordaClient) {
        Map<String, HoldingIdentity> map = new HashMap<>();

        try {
            List<VirtualNode> vNodeList = cordaClient.getVirtualNodeList();

            for (VirtualNode vNode : vNodeList) {
                var identity = vNode.holdingIdentity();

                if (identity.isPlayer()) {
                    map.put(identity.getName().toLowerCase(locale), identity);
                }
            }
        } catch (InvalidNameException e) {
            // TODO: logs
            System.out.println("Unable to get ShorHash map for Corda virtual nodes: "+e.getExplanation());
            e.printStackTrace();
        }

        return map;
    }

    /*
     * @param apiUserName 
     *  HoldingIdentity x500 name typically looks like 
     *  "CN=Bob, OU=Player, O=Checkers, L=Kviv, C=UA"
     *  CN - is a common name, expected to be unique for CordaCheckers setup.
     * 
     * @return HoldingIdentity
     */
    public HoldingIdentity getByCommonName(String apiUserName) {
        return this.cache.get(apiUserName.toLowerCase(locale));
    }
}
